package jp.programmingmat.www.gbtkgl10;

import java.util.ArrayList;

import javax.microedition.khronos.opengles.GL10;

/**
 * 複数のmeshをまとめて一つの物体として扱う複合meshクラス。
 * 複雑な構造を持つ独自の物体を定義する場合のベースクラスとしても使用できます。
 */
public class GbtkGL10Object implements GbtkGL10IDrawableObject {

	private ArrayList<GbtkGL10Mesh> mMeshList;

	private GbtkGL10Transform mTransform;
	
	public GbtkGL10Object() {
		
		mMeshList = new ArrayList<GbtkGL10Mesh>();

		mTransform = new GbtkGL10Transform();
		
	}

	public GbtkGL10Object(GbtkGL10Mesh mesh) {

		mMeshList = new ArrayList<GbtkGL10Mesh>();

		mTransform = new GbtkGL10Transform();
		
		addMesh(mesh);

	}

	public GbtkGL10Mesh getMesh(int index) {

		if (index < 0 || index >= mMeshList.size()) {
			return null;
		}
		
		return mMeshList.get(index);
		
	}

	/**
	 * meshを追加します。
	 * @param mesh 追加するmesh
	 */
	public void addMesh(GbtkGL10Mesh mesh) {

		if (mesh == null) {
			return;
		}
		
		mMeshList.add(mesh);
		
	}

	/**
	 * 物体に設定されている位置を取得します。
	 * @return 物体の位置（x/y/z）を保持するベクトル
	 */
	public GbtkGL10Vector3f getPosition() {
		return mTransform.getTranslate();
	}

	/**
	 * 物体の位置を設定します。
	 * @param x X座標
	 * @param y Y座標
	 * @param z Z座標
	 */
	public void setPosition(float x, float y, float z) {
		mTransform.setTranslate(x,  y,  z);
	}
	
	public GbtkGL10Vector3f getRotate() {
		return mTransform.getRotate();
	}
	
	public void setRotate(float rx, float ry, float rz) {
		mTransform.setRotate(rx, ry, rz);
	}

	public void move(float dx, float dy, float dz) {
		mTransform.addTranslate(dx, dy, dz);
	}
	
	public void addRotate(float dx, float dy, float dz) {
		mTransform.addRotate(dx, dy, dz);
	}

	public void draw(GbtkGL10World world) {
		draw(world, 0, 0, 0);
	}

	public void draw(GbtkGL10World world, GbtkGL10Vector3f position) {

		if (position == null) {
			return;
		}
		
		draw(world, position.getFloatArray());

	}

	public void draw(GbtkGL10World world, float[] position) {

		if (position == null || position.length < 3) {
			return;
		}

		draw(world, position[0], position[1], position[2]);
		
	}
	
	public void draw(GbtkGL10World world, float dx, float dy, float dz) {
		
		if (mMeshList.size() < 1) {
			return;
		}

		GL10 gl = world.getGLInterface();
		
		gl.glPushMatrix();
		
		gl.glTranslatef(mTransform.getTranslate().get(0) + dx, mTransform.getTranslate().get(1) + dy, mTransform.getTranslate().get(2) + dz);

		gl.glRotatef(mTransform.getRotate().get(0), 1, 0, 0);
		gl.glRotatef(mTransform.getRotate().get(1), 0, 1, 0);
		gl.glRotatef(mTransform.getRotate().get(2), 0, 0, 1);
		
		for (int i = 0;i < mMeshList.size();i++) {
			mMeshList.get(i).draw(world);
		}

		gl.glPopMatrix();
		
	}

	public void draw(GbtkGL10World world, float dx, float dy, float dz, float rx, float ry, float rz) {
		
		if (mMeshList.size() < 1) {
			return;
		}

		GL10 gl = world.getGLInterface();
		
		gl.glPushMatrix();
		
		gl.glTranslatef(mTransform.getTranslate().get(0) + dx, mTransform.getTranslate().get(1) + dy, mTransform.getTranslate().get(2) + dz);

		gl.glRotatef(mTransform.getRotate().get(0) + rx, 1, 0, 0);
		gl.glRotatef(mTransform.getRotate().get(1) + ry, 0, 1, 0);
		gl.glRotatef(mTransform.getRotate().get(2) + rz, 0, 0, 1);
		
		for (int i = 0;i < mMeshList.size();i++) {
			mMeshList.get(i).draw(world);
		}

		gl.glPopMatrix();
		
	}

	public void draw(GbtkGL10World world, GbtkGL10Transform transform) {

		if (mMeshList.size() < 1) {
			return;
		}

		if (transform == null) {
			draw(world);
		} else {
			draw(world, transform.getTranslate().get(0), transform.getTranslate().get(1), transform.getTranslate().get(2), transform.getRotate().get(0), transform.getRotate().get(1), transform.getRotate().get(2));
		}

	}
	
}