package jp.programmingmat.www.gbtkgl10;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * 同一オブジェクトを複数同時描画する描画管理クラス。
 * 一度の頂点データ転送で位置を変えながら複数の物体を描画するため、描画時の効率向上が期待できます。
 */
public class GbtkGL10DrawableObjectDuplicator implements GbtkGL10IDrawableObject {

	private ArrayList<GbtkGL10Transform> mDrawingTransformList;

	private GbtkGL10IDrawableObject mDrawableObject;

	public GbtkGL10DrawableObjectDuplicator() {

		mDrawingTransformList = new ArrayList<GbtkGL10Transform>();

		mDrawableObject = null;
		
	}

	public GbtkGL10DrawableObjectDuplicator(GbtkGL10IDrawableObject obj) {

		mDrawingTransformList = new ArrayList<GbtkGL10Transform>();

		mDrawableObject = obj;
		
	}

	/**
	 * 現在オブジェクトが保持している描画位置情報の数（オブジェクト描画数）を取得します。
	 * @return 描画位置情報の数
	 */
	public int length() {
		return mDrawingTransformList.size();
	}

	/**
	 * 描画対象となっているオブジェクトを取得します。
	 * @return 描画対象オブジェクト
	 */
	public GbtkGL10IDrawableObject getDrawableObject() {
		return mDrawableObject;
	}

	/**
	 * 描画対象とするオブジェクトを設定します。
	 * @param obj 描画対象オブジェクト
	 */
	public void setDrawableObject(GbtkGL10IDrawableObject obj) {
		mDrawableObject = obj;
	}
	
	/**
	 * 描画位置設定を追加（位置のみで回転なし）。
	 * @param x X座標
	 * @param y Y座標
	 * @param z Z座標
	 */
	public void addDrawTransform(float x, float y, float z) {
		mDrawingTransformList.add(new GbtkGL10Transform(x, y, z, 0, 0, 0));
	}

	/**
	 * 描画位置設定を追加します。
	 * @param trans 描画位置設定（位置と回転を保持するtransform）
	 */
	public void addDrawTransform(GbtkGL10Transform trans) {
		
		if (trans == null) {
			return;
		}

		addDrawTransform(trans);
		
	}
	
	/**
	 * 指定番号の描画位置設定を取得します。
	 * @param index 描画ポイントの番号（リスト先頭を0とするインデックス）
	 */
	public GbtkGL10Transform getDrawTransform(int index) {

		if (index < 0 || index >= mDrawingTransformList.size()) {
			return null;
		}
		
		return mDrawingTransformList.get(index);
		
	}

	/**
	 * 指定番号の描画位置設定を削除します。
	 * @param index 描画ポイントの番号（リスト先頭を0とするインデックス）
	 */
	public void removeDrawTransform(int index) {

		if (index < 0 || index >= mDrawingTransformList.size()) {
			return;
		}
		
		mDrawingTransformList.remove(index);
		
	}

	/**
	 * OpenGL ESでの描画時にシステムが呼び出す描画関数です。
	 */
	public void draw(GbtkGL10World world) {
		draw(world, 0, 0, 0);
	}

	public void draw(GbtkGL10World world, GbtkGL10Vector3f position) {
		
		if (position == null) {
			draw(world, 0, 0, 0);
		} else {
			draw(world, position.get(0), position.get(1), position.get(2));
		}
		
	}
	
	public void draw(GbtkGL10World world, float dx, float dy, float dz) {

		if (mDrawableObject == null) {
			return;
		}

		Iterator<GbtkGL10Transform> i = mDrawingTransformList.iterator();

		while (i.hasNext()) {
			
			GbtkGL10Transform trans = i.next();
			
			mDrawableObject.draw(world, trans.getTranslate().get(0) + dx, trans.getTranslate().get(1) + dy, trans.getTranslate().get(2) + dz, trans.getRotate().get(0), trans.getRotate().get(1), trans.getRotate().get(2));

		}
		
	}

	public void draw(GbtkGL10World world, float dx, float dy, float dz, float rx, float ry, float rz) {

		if (mDrawableObject == null) {
			return;
		}

		Iterator<GbtkGL10Transform> i = mDrawingTransformList.iterator();

		while (i.hasNext()) {
			
			GbtkGL10Transform trans = i.next();

			float[] translate = trans.getTranslate().getFloatArray();
			float[] rotate = trans.getRotate().getFloatArray();
			
			mDrawableObject.draw(world, translate[0] + dx, translate[1] + dy, translate[2] + dz, rotate[0] + rx, rotate[1] + ry, rotate[2] + rz);

		}
		
	}
	
	public void draw(GbtkGL10World world, GbtkGL10Transform transform) {

		if (mDrawableObject == null) {
			return;
		}

		Iterator<GbtkGL10Transform> i = mDrawingTransformList.iterator();

		while (i.hasNext()) {
			
			GbtkGL10Transform trans = i.next();

			float[] translate = trans.getTranslate().getFloatArray();
			float[] rotate = trans.getRotate().getFloatArray();
			
			mDrawableObject.draw(world, translate[0] + transform.getTranslate().get(0), translate[1] + transform.getTranslate().get(1), translate[2] + transform.getTranslate().get(2), rotate[0] + transform.getRotate().get(0), rotate[1] + transform.getRotate().get(1), rotate[2] + transform.getRotate().get(2));

		}
		
	}

}
