package jp.programmingmat.www.gbtkgl10;

import javax.microedition.khronos.opengles.GL10;

/**
 * 描画時の表面材質データ（色、テクスチャ情報）を保持するmaterialクラス。
 */
public class GbtkGL10Material {

	private float[] mDiffuseRGBA;
	private float[] mAmbientRGBA;
	private float[] mSpecularRGBA;
	private float[] mEmissionRGBA;

	private float[] mDefaultDiffuseRGBA = {0.5f, 0.5f, 0.5f, 1.0f};
	private float[] mDefaultAmbientRGBA = {0.5f, 0.5f, 0.5f, 1.0f};
	private float[] mDefaultSpecularRGBA = {0, 0, 0, 1};
	private float[] mDefaultEmissionRGBA = {0, 0, 0, 1};
	
	private int mShininess;
	
	private int mTextureIndex;
	
	private float mTextureUDiv, mTextureVDiv;
	
	public GbtkGL10Material() {

		mTextureIndex = -1;
		
	}

	/**
	 * 表面色を指定してmaterialを作成します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public GbtkGL10Material(float r, float g, float b, float a) {

		setRGBA(r, g, b, a);

		mTextureIndex = -1;
		
	}

	/**
	 * 表面色とテクスチャを指定してmaterialを作成します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 * @param textureIndex テクスチャ番号
	 */
	public GbtkGL10Material(float r, float g, float b, float a, int textureIndex) {

		setRGBA(r, g, b, a);

		setTexture(textureIndex);
		
	}
	
	/**
	 * 色を指定します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setRGBA(float r, float g, float b, float a) {
	
		setDiffuseRGBA(r, g, b, a);
		setAmbientRGBA(r, g, b, a);
		
	}
	
	/**
	 * 拡散反射色を個別に指定します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setDiffuseRGBA(float r, float g, float b, float a) {
		mDiffuseRGBA = new float[] {r, g, b, a};
	}

	/**
	 * 環境反射色を個別に指定します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setAmbientRGBA(float r, float g, float b, float a) {
		mAmbientRGBA = new float[] {r, g, b, a};
	}

	/**
	 * 鏡面反射色を設定します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setSpecularRGBA(float r, float g, float b, float a) {
		mSpecularRGBA = new float[] {r, g, b, a};
	}

	/**
	 * 放射色を設定します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setEmissionRGBA(float r, float g, float b, float a) {
		mEmissionRGBA = new float[] {r, g, b, a};
	}
	
	/**
	 * 鏡面光指数を設定します。
	 * @param shininess 鏡面光指数
	 */
	public void setShininess(int shininess) {
		mShininess = shininess;
	}

	public float getTextureUDiv() {
		return mTextureUDiv;
	}

	public float getTextureVDiv() {
		return mTextureVDiv;
	}

	/**
	 * テクスチャを指定します。
	 * @param index テクスチャ番号
	 */
	public void setTexture(int index) {
		setTexture(index, 1, 1);
	}
	
	/**
	 * テクスチャを指定します。
	 * @param index テクスチャ番号
	 * @param udiv U方向の繰り返し数
	 * @param vdiv V方向の繰り返し数
	 */
	public void setTexture(int index, float udiv, float vdiv) {

		mTextureIndex = index;

		mTextureUDiv = udiv;
		mTextureVDiv = vdiv;
		
	}

	/**
	 * 保持している表面材質情報を出力します。
	 * システムがOpenGL ESによる描画を行う時に、自動的に呼び出されます。
	 * @param world
	 * @param geo
	 */
	public void putMaterial(GbtkGL10World world, GbtkGL10Geometry geo) {

		GL10 gl = world.getGLInterface();
		
		if (mDiffuseRGBA != null) {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_DIFFUSE, mDiffuseRGBA, 0);
		} else {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_DIFFUSE, mDefaultDiffuseRGBA, 0);
		}

		if (mAmbientRGBA != null) {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_AMBIENT, mAmbientRGBA, 0);
		} else {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_AMBIENT, mDefaultAmbientRGBA, 0);
		}
		
		if (mSpecularRGBA != null) {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_SPECULAR, mSpecularRGBA, 0);
		} else {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_SPECULAR, mDefaultSpecularRGBA, 0);
		}

		if (mEmissionRGBA != null) {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_EMISSION, mEmissionRGBA, 0);
		} else {
			gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_EMISSION, mDefaultEmissionRGBA, 0);
		}
		
		gl.glMaterialx(GL10.GL_FRONT_AND_BACK, GL10.GL_SHININESS, mShininess);

		// テクスチャ情報があれば、テクスチャを有効化して設定
		if (mTextureIndex != -1 && geo.getTexturePointBuffer() != null) {
			
			gl.glEnable(GL10.GL_TEXTURE_2D);

			gl.glBindTexture(GL10.GL_TEXTURE_2D, world.getTexture(mTextureIndex));
			
			gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
			gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, geo.getTexturePointBuffer());
			
		} else {
			gl.glDisable(GL10.GL_TEXTURE_2D);
		}
		
	}
	
}
