package jp.programmingmat.www.gbtkgl10;

import java.util.ArrayList;

import javax.microedition.khronos.opengles.GL10;

/**
 * 図形（頂点）と表面材質をまとめた描画可能な物体クラス。
 * worldに追加して描画することが出来る基本単位（レンダリングの構成要素）です。
 */
public class GbtkGL10Mesh implements GbtkGL10IDrawableObject {

	private GbtkGL10Material mMaterial;
	private ArrayList<GbtkGL10Geometry> mGeometryList;
	
	private GbtkGL10Transform mTransform;
	
	public GbtkGL10Mesh() {

		mGeometryList = new ArrayList<GbtkGL10Geometry>();

		mTransform = new GbtkGL10Transform();
		
	}

	/**
	 * geometryとmaterialを指定してmeshを作成します。
	 * @param geometry
	 * @param material
	 */
	public GbtkGL10Mesh(GbtkGL10Geometry geometry, GbtkGL10Material material) {

		mGeometryList = new ArrayList<GbtkGL10Geometry>();

		mTransform = new GbtkGL10Transform();
		
		geometry.setTexturePointBuffer(material.getTextureUDiv(), material.getTextureVDiv());
		
		addGeometry(geometry);
		setMaterial(material);
		
	}
	
	public void addGeometry(GbtkGL10Geometry geometry) {

		if (geometry == null) {
			return;
		}
		
		mGeometryList.add(geometry);
		
	}

	/**
	 * 設定されているmateerialを取得します。
	 * @return material
	 */
	public GbtkGL10Material getMaterial() {
		return mMaterial;
	}

	/**
	 * materialを設定します。
	 * @param material
	 */
	public void setMaterial(GbtkGL10Material material) {
		mMaterial = material;
	}

	public GbtkGL10Vector3f getPosition() {
		return mTransform.getTranslate();
	}
	
	public void setPosition(float x, float y, float z) {
		mTransform.setTranslate(x, y, z);
	}
	
	public GbtkGL10Vector3f getRotate() {
		return mTransform.getRotate();
	}
	
	/**
	 * 物体の回転角度を設定します。
	 * @param rx X軸を基準とする回転角度
	 * @param ry Y軸を基準とする回転角度
	 * @param rz Z軸を基準とする回転角度
	 */
	public void setRotate(float rx, float ry, float rz) {
		mTransform.setRotate(rx, ry, rz);
	}

	/**
	 * 物体の回転角度に指定値を加算します。
	 * @param dx X軸を基準とする回転角度の加算値
	 * @param dy Y軸を基準とする回転角度の加算値
	 * @param dz Z軸を基準とする回転角度の加算値
	 */
	public void addRotate(float dx, float dy, float dz) {
		mTransform.addRotate(dx, dy, dz);
	}
	
	public void draw(GbtkGL10World world, GbtkGL10Vector3f position) {

		if (position == null) {
			return;
		}
		
		draw(world, position.getFloatArray());

	}

	public void draw(GbtkGL10World world, float[] position) {

		if (position == null || position.length < 3) {
			return;
		}

		draw(world, position[0], position[1], position[2]);
		
	}
	
	public void draw(GbtkGL10World world) {
		draw(world, 0, 0, 0);
	}
	
	public void draw(GbtkGL10World world, float dx, float dy, float dz) {
		
		if (mGeometryList.size() < 1) {
			return;
		}

		GL10 gl = world.getGLInterface();
		
		gl.glPushMatrix();
		
		gl.glTranslatef(mTransform.getTranslate().get(0) + dx, mTransform.getTranslate().get(1) + dy, mTransform.getTranslate().get(2) + dz);

		gl.glRotatef(mTransform.getRotate().get(0), 1, 0, 0);
		gl.glRotatef(mTransform.getRotate().get(1), 0, 1, 0);
		gl.glRotatef(mTransform.getRotate().get(2), 0, 0, 1);
		
		for (int i = 0;i < mGeometryList.size();i++) {
			mGeometryList.get(i).draw(world, mMaterial);
		}
		
		gl.glPopMatrix();

	}

	public void draw(GbtkGL10World world, float dx, float dy, float dz, float rx, float ry, float rz) {
		
		if (mGeometryList.size() < 1) {
			return;
		}

		GL10 gl = world.getGLInterface();
		
		gl.glPushMatrix();
		
		gl.glTranslatef(mTransform.getTranslate().get(0) + dx, mTransform.getTranslate().get(1) + dy, mTransform.getTranslate().get(2) + dz);

		gl.glRotatef(mTransform.getRotate().get(0) + rx, 1, 0, 0);
		gl.glRotatef(mTransform.getRotate().get(1) + ry, 0, 1, 0);
		gl.glRotatef(mTransform.getRotate().get(2) + rz, 0, 0, 1);
		
		for (int i = 0;i < mGeometryList.size();i++) {
			mGeometryList.get(i).draw(world, mMaterial);
		}
		
		gl.glPopMatrix();

	}

	public void draw(GbtkGL10World world, GbtkGL10Transform transform) {

		if (mGeometryList.size() < 1) {
			return;
		}

		if (transform == null) {
			draw(world);
		} else {
			draw(world, transform.getTranslate().get(0), transform.getTranslate().get(1), transform.getTranslate().get(2), transform.getRotate().get(0), transform.getRotate().get(1), transform.getRotate().get(2));
		}

	}
	
}