package jp.programmingmat.www.gbtkgl10;

import java.util.ArrayList;
import java.util.Iterator;

import javax.microedition.khronos.opengles.GL10;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.opengl.GLUtils;
import android.os.SystemClock;

/**
 * アプリのデータ・状態を管理するモデルクラス。
 * init()でデータを作成し、loadTexture()でテクスチャを設定。その後process()で状態の更新を行ないます。
*/
public class GbtkGL10World {

	private GL10 mGLInterface;

	private int[] mTextureIDList;
	
	public static final int INDEXED_TEXTURE_NUM = 256;
	
	private ArrayList<GbtkGL10IDrawableObject> mDrawObjectList;

	private float[] mGlobalLightVector;
	
	private float[] mGlobalLightDiffuseRGBA;
	private float[] mGlobalLightAmbientRGBA;
	private float[] mGlobalLightSpecularRGBA;
	
	private GbtkGL10Camera mCamera;
	private float mAspect;
	
	private float[] mClearRGBA;
	
	private GbtkGL10Activity mActivity;
	
	private long mCreatedMS;
	
	public GbtkGL10World(GbtkGL10Activity activity) {

		// 生成時間（ブート時からのシステム時間）を記録
		mCreatedMS = SystemClock.elapsedRealtime();
		 
		mActivity = activity;
		
		mDrawObjectList = new ArrayList<GbtkGL10IDrawableObject>();
	
		mCamera = new GbtkGL10Camera();
		mAspect = 1.0f;

		mTextureIDList = new int[INDEXED_TEXTURE_NUM];

		// グローバルライトのデフォルト光源パラメーターを設定
		setGlobalLightVector(12.0f, 10.0f, 10.0f);
		setGlobalLightRGBA(1.0f, 1.0f, 1.0f, 1.0f);
		
		setClearRGBA(0.0f, 0.0f, 0.0f, 1.0f);
		
	}

	/**
	 * world構築時に呼び出される初期化関数。
	 * サブクラスで、モデルデータの初期化などの処理を記述してください。
	 */
	public void init() {
	}

	/**
	 * OpenGL ESシステムの初期化（再構築）時に呼び出されるOpenGL ES関連リソース生成関数。
	 * サブクラスでテクスチャ読み込みなどOpenGL ESの初期化に関する処理を記述してください。アプリ起動時だけでなく、ポーズからの復帰時などにも呼び出される可能性があります。
	 */
	public void loadTexture() {
	}

	/**
	 * world構築時からの経過時間をミリ秒単位で取得。
	 * @return world構築時からの経過時間（ミリ秒単位）
	 */
	public long getWorldTimeMillis() {
		return SystemClock.elapsedRealtime() - mCreatedMS;
	}

	/**
	 * アプリのActivityを取得。
	 * process()は、UIスレッドとは別スレッドで実行されているので、process()から直接Activityを操作しないようにしてください。
	 * @return アプリのActivity
	 */
	protected GbtkGL10Activity getActivity() {
		return mActivity;
	}
	
	protected Resources getResources() {
		return mActivity.getResources();
	}
	
	public GbtkGL10Camera getCamera() {
		return mCamera;
	}

	/**
	 * 画面クリアの色（背景色）を設定。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setClearRGBA(float r, float g, float b, float a) {
		mClearRGBA = new float[]{r, g, b, a};
	}
	
	public void setAspect(float aspect) {
		
		mAspect = aspect;
		
		mCamera.setAspect(mAspect);

	}
	
	public Iterator<GbtkGL10IDrawableObject> getObjectIterator() {
		return mDrawObjectList.iterator();
	}

	/**
	 * worldで管理（描画）する物体を追加。
	 * @param obj 追加する物体
	 */
	public void addDrawObject(GbtkGL10IDrawableObject obj) {

		if (obj == null) {
			return;
		}
		
		mDrawObjectList.add(obj);
		
	}
	
	public GL10 getGLInterface() {
		return mGLInterface;
	}

	public void setGLInterface(GL10 gl) {
		mGLInterface = gl;
	}

	/**
	 * 指定番号のテクスチャのOpenGL ES内部におけるIDを取得します。
	 * システムがOpenGL ESでの描画時に呼び出す関数です。
	 * @param index テクスチャ番号
	 * @return OpenGL ESで使用するテクスチャID
	 */
	public int getTexture(int index) {

		if (index < 0 || index >= INDEXED_TEXTURE_NUM) {
			return 0;
		}
		
		return mTextureIDList[index];

	}

	/**
	 * テクスチャ登録。
	 * この関数で登録したテクスチャは、マテリアルで番号（index）を指定する形で使用します。
	 * @param bmp テクスチャの画像として使用するBitmap
	 * @param index テクスチャ番号
	*/
	public void setTexture(Bitmap bmp, int index) {

		if (bmp == null || index < 0 || index >= INDEXED_TEXTURE_NUM) {
			return;
		}
		
		GL10 gl = getGLInterface();
		
		gl.glEnable(GL10.GL_TEXTURE_2D);
		
		// GLES側の管理番号を保存
		gl.glGenTextures(1, mTextureIDList, index);

		// 取得した管理番号のテクスチャを操作対象に設定
		gl.glBindTexture(GL10.GL_TEXTURE_2D, mTextureIDList[index]);

		// ビットマップをテクスチャとして設定
		GLUtils.texImage2D(GL10.GL_TEXTURE_2D, 0, bmp, 0);

		// 繰り返し設定
        gl.glTexParameterx(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_WRAP_S, GL10.GL_REPEAT );
        gl.glTexParameterx(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_WRAP_T, GL10.GL_REPEAT );

        // 拡大縮小時の処理設定
        gl.glTexParameterx(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER, GL10.GL_LINEAR );
        gl.glTexParameterx(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER, GL10.GL_LINEAR );

		gl.glBindTexture(GL10.GL_TEXTURE_2D, 0);
        
	}

	/**
	 * グローバルライトの向きを設定。
	 * @param x 向きベクトルのX成分
	 * @param y 向きベクトルのY成分
	 * @param z 向きベクトルのZ成分
	 */
	public void setGlobalLightVector(float x, float y, float z) {
		mGlobalLightVector = new float[]{x, y, z, 0};
	}

	/**
	 * グローバルライトの色を設定。
	 * 鏡面光、拡散光、環境光を（それぞれ調整の上）まとめて設定します。
	 * @param r 赤成分
	 * @param g 緑成分
	 * @param b 青成分
	 * @param a アルファ値
	 */
	public void setGlobalLightRGBA(float r, float g, float b, float a) {

		mGlobalLightSpecularRGBA = new float[]{r, g, b, a};
		mGlobalLightDiffuseRGBA = new float[]{r * 0.7f, g * 0.7f, b * 0.7f, a};
		mGlobalLightAmbientRGBA = new float[]{r * 0.3f, g * 0.3f, b * 0.3f, a};
		
	}

	public void setGlobalLightDiffuseRGBA(float r, float g, float b, float a) {
		mGlobalLightDiffuseRGBA = new float[]{r, g, b, a};
	}

	public void setGlobalLightAmbientRGBA(float r, float g, float b, float a) {
		mGlobalLightAmbientRGBA = new float[]{r, g, b, a};
	}

	public void setGlobalLightSpecularRGBA(float r, float g, float b, float a) {
		mGlobalLightSpecularRGBA = new float[]{r, g, b, a};
	}
	
	/**
	 * グローバルライト情報を出力。
	 * OpenGL ESでの描画時にシステムが呼び出す関数です。
	 */
	public void putGlobalLight() {

		mGLInterface.glEnable(GL10.GL_LIGHTING);
		mGLInterface.glEnable(GL10.GL_LIGHT0);
		
		mGLInterface.glLightfv(GL10.GL_LIGHT0, GL10.GL_DIFFUSE, mGlobalLightDiffuseRGBA, 0);
		mGLInterface.glLightfv(GL10.GL_LIGHT0, GL10.GL_AMBIENT, mGlobalLightAmbientRGBA, 0);
		mGLInterface.glLightfv(GL10.GL_LIGHT0, GL10.GL_SPECULAR, mGlobalLightSpecularRGBA, 0);
		
		mGLInterface.glLightfv(GL10.GL_LIGHT0,GL10.GL_POSITION, mGlobalLightVector, 0);
		
	}

	/**
		モデル更新関数。
		この関数をオーバーライドする形で各オブジェクトの状態更新を行なってください
	*/
	public void process() {
	}

	/**
		GLSurfaceViewの描画にあわせて呼ばれる描画イベントハンドラ。
		デフォルトでは、ここからモデル更新関数process()を呼び出します
	*/
	public void prepareRender() {
		process();
	}

	/**
	 * 描画関数
	 */
	public void render() {

		GL10 gl = getGLInterface();

		gl.glClearColor(mClearRGBA[0], mClearRGBA[1], mClearRGBA[2], mClearRGBA[3]);

    	gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);

        gl.glEnable(GL10.GL_DEPTH_TEST);
    	gl.glShadeModel(GL10.GL_SMOOTH);

    	// カメラ情報を設定
    	mCamera.setCamera(this);

    	putGlobalLight();

    	// 保持しているオブジェクトのイテレーターを取得
    	Iterator<GbtkGL10IDrawableObject> i = getObjectIterator();

    	// 各オブジェクトを描画
		while (i.hasNext()) {

			// モデル配置座標を初期化
	    	gl.glMatrixMode(GL10.GL_MODELVIEW);
	    	gl.glLoadIdentity();

			GbtkGL10IDrawableObject obj = i.next();
			
			obj.draw(this);

		}

	}
	
}